﻿using UnityEngine;

[ExecuteInEditMode]
public class DrawMeshProcedural : MonoBehaviour
{
    public Mesh mesh;
    public Texture tex;
    public Material mat;
    
    private ComputeBuffer meshUVs;
    private ComputeBuffer meshIndices;
    private ComputeBuffer meshNormals;
    private ComputeBuffer meshVertices;

    void OnRenderObject()
    {
        if (Camera.current == null) return;

        CreateResources();

        mat.SetTexture("_Texture", tex);
        mat.SetBuffer("_MeshUVs", meshUVs);
        mat.SetBuffer("_MeshIndices", meshIndices);
        mat.SetBuffer("_MeshNormals", meshNormals);
        mat.SetBuffer("_MeshVertices", meshVertices);

        mat.SetPass(0);
        Graphics.DrawProcedural(MeshTopology.Triangles, meshIndices.count);
    }

    void CreateResources()
    {
        if (meshUVs == null)
        {
            var uvs = mesh.uv;
            meshUVs = new ComputeBuffer(uvs.Length, sizeof(float) * 2);
            meshUVs.SetData(uvs);
        }

        if (meshIndices == null)
        {
            var indices = mesh.triangles;
            meshIndices = new ComputeBuffer(indices.Length, sizeof(float) * 1);
            meshIndices.SetData(indices);
        }

        if (meshNormals == null)
        {
            var normals = mesh.normals;
            meshNormals = new ComputeBuffer(normals.Length, sizeof(float) * 3);
            meshNormals.SetData(normals);
        }

        if (meshVertices == null)
        {
            var vertices = mesh.vertices;
            meshVertices = new ComputeBuffer(vertices.Length, sizeof(float) * 3);
            meshVertices.SetData(vertices);
        }
    }

    void OnDisable()
    {
        if (meshUVs != null)
        {
            meshUVs.Release();
            meshUVs = null;
        }
        if (meshIndices != null)
        {
            meshIndices.Release();
            meshIndices = null;
        }

        if (meshNormals != null)
        {
            meshNormals.Release();
            meshNormals = null;
        }

        if (meshVertices != null)
        {
            meshVertices.Release();
            meshVertices = null;
        }
    }
}
