﻿using UnityEngine;

[ExecuteInEditMode]
public class DrawRandomProcedural : MonoBehaviour
{
    public Material mat;

    public int NumVertices;
    public int SpawnSizeRegion;

    private ComputeBuffer randomColors;
    private ComputeBuffer randomVertices;

    void OnRenderObject()
    {
        if (Camera.current == null) return;

        CreateResources();

        mat.SetBuffer("_RandomColors", randomColors);
        mat.SetBuffer("_RandomVertices", randomVertices);

        mat.SetPass(0);
        Graphics.DrawProcedural(MeshTopology.Triangles, NumVertices);
    }

    void CreateResources()
    {
        if (randomVertices == null)
        {
            var positions = new Vector3[NumVertices];
            for (int i = 0; i < NumVertices; i++)
            {
                positions[i] = Random.insideUnitSphere * SpawnSizeRegion;
            }

            randomVertices = new ComputeBuffer(NumVertices, sizeof(float) * 3);
            randomVertices.SetData(positions);
        }

        if (randomColors == null)
        {
            var colors = new Color[NumVertices];
            for (int i = 0; i < NumVertices; i++)
            {
                colors[i] = new Color(Random.value, Random.value, Random.value);
            }

            randomColors = new ComputeBuffer(NumVertices, sizeof(float) * 4);
            randomColors.SetData(colors);
        }
    }

    void OnDisable()
    {
        if (randomColors != null)
        {
            randomColors.Release();
            randomColors = null;
        }

        if (randomVertices != null)
        {
            randomVertices.Release();
            randomVertices = null;
        }        
    }
}
