﻿Shader "Custom/DrawBillboards"
{
	Properties
	{
		_MainTex ("Texture", 2D) = "white" {}
	}
		SubShader
	{
		Tags { "RenderType" = "Opaque" }
		LOD 100

		Pass
		{	
			Cull Off
			//Blend OneMinusDstColor One // Soft Additive
			//Blend SrcAlpha OneMinusSrcAlpha // Alpha blending

			//Cull Off
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag

			#include "UnityCG.cginc"

			float4 _CameraUp;
			float4 _CameraRight;

			sampler2D _Texture;

			StructuredBuffer<float2> _QuadUVs;
			StructuredBuffer<int> _QuadIndices;
			StructuredBuffer<float3> _QuadVertices;
			StructuredBuffer<float4> _InstanceColors;
			StructuredBuffer<float3> _InstancePositions;

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 color : COLOR0;
				float4 vertex : SV_POSITION;
			};
			
			v2f vert(uint vertexId : SV_VertexID, uint instanceId : SV_InstanceID)
			//v2f vert(appdata_base v)
			{
				int index = _QuadIndices[vertexId];
				float3 vertexPos = _QuadVertices[index];
				
				float3 billboardWorldPos = _InstancePositions[instanceId];
				float4 billboardVertexWorldPos = float4(billboardWorldPos,0) + _CameraRight * vertexPos.x + _CameraUp * vertexPos.y;

				v2f output;
				output.uv = _QuadUVs[index];
				output.color = _InstanceColors[instanceId];
				//output.vertex = mul(UNITY_MATRIX_MVP, float4(vertexPos.xyz, 1));
				output.vertex = mul(UNITY_MATRIX_MVP, float4(billboardVertexWorldPos.xyz,1));
				return output;
			}
			
			float4 frag (v2f i) : SV_Target
			{
				//return float4(1,1,1,1);
				float4 color = tex2D(_Texture, i.uv) * i.color * 1.25;				
				if (color.a <= 0.1) discard;
				return color;
			}
			ENDCG
		}
	}
}
