﻿Shader "Custom/DrawInstanced"
{
	SubShader
	{
		Tags{ "RenderType" = "Opaque" }
		LOD 100

		Pass
		{		
			Cull Back

			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag

			#include "UnityCG.cginc"

			sampler2D _Texture;

			StructuredBuffer<float2> _MeshUVs;
			StructuredBuffer<int> _MeshIndices;
			StructuredBuffer<float3> _MeshNormals;
			StructuredBuffer<float3> _MeshVertices;
			StructuredBuffer<float4> _InstanceColors;
			StructuredBuffer<float4x4> _InstanceMatrices;

			struct v2f
			{
				float3 lightDir : FLOAT30;
				float3 normal : FLOAT31;

				float2 uv : TEXCOORD0;
				float4 color : COLOR0;
				float4 vertex : SV_POSITION;
			};

			v2f vert(uint vertexId : SV_VertexID, uint instanceId : SV_InstanceID)
			{
				int index = _MeshIndices[vertexId];
				float3 normal = _MeshNormals[index];
				float3 vertex = _MeshVertices[index];
				float4x4 modelMatrix = _InstanceMatrices[instanceId];

				float4 worldPos = mul(modelMatrix, float4(vertex, 1));
				float3 worldNormal = mul(modelMatrix, float4(normal, 0));
				float3 lightDir = normalize(_WorldSpaceCameraPos - mul(modelMatrix, float4(vertex, 1)));

				v2f output;
				output.uv = _MeshUVs[index]; 
				output.color = _InstanceColors[instanceId];
				output.vertex = mul(UNITY_MATRIX_VP, worldPos);
				output.normal = worldNormal;
				output.lightDir = lightDir;
				return output;
			}

			float4 frag(v2f i) : SV_Target
			{
				float light = max(dot(normalize(i.normal), normalize(i.lightDir)), 0);
				return float4(1,1,1,1) * light * i.color;
				//return tex2D(_Texture, i.uv);
			}
			ENDCG
		}
	}
}
